package Mac::MoreFiles::_ApplHash;

package Mac::MoreFiles;

=head1 NAME

Mac::MoreFiles - Macintosh Interface to MoreFiles application notes

=head1 SYNOPSIS


	use Mac::MoreFiles;
	
	$application = $Application{"MrPL"};

=head1 DESCRIPTION

=head2 %Application

The %Application hash will fetch the application FSSpec for a given signature,
searching on all mounted volumes.

=include MF.xs

=head1 AUTHOR(S)

Matthias Ulrich Neeracher <neeri@iis.ee.ethz.ch> Author

Bob Dalgleish <bob.dalgleish@sasknet.sk.ca> Documenter

=cut

require Exporter;
require DynaLoader;
@ISA = (Exporter, DynaLoader);
@EXPORT = qw(
	FSpCreateMinimum
	FSpShare
	FSpUnshare
	FSpFileCopy
	FSpDirectoryCopy
	FSpIterateDirectory
	FSpDTGetAPPL
	FSpDTSetComment
	FSpDTGetComment
	FSpDTCopyComment
	%Application
);

package Mac::MoreFiles::_ApplHash;

require Tie::Hash;

@ISA = qw(Tie::StdHash);

sub FETCH {
	my($self,$id) = @_;
	my($vol,$app);
	
	if (!$self->{$id}) {
		for $vol (&MacPerl::Volumes) {
			$app = Mac::MoreFiles::FSpDTGetAPPL(hex(substr($vol, 1, 4)), $id);
			last if ($app);
		}
		$self->{$id} = $app;
	}
	$self->{$id};
}

package Mac::MoreFiles;

tie %Application, Mac::MoreFiles::_ApplHash;

bootstrap Mac::MoreFiles;

1;

__END__
