/*==============================================================================
Project:	POV

Version:	3

File:	LnkLst.h

Description:
Routines to handle linked lists
------------------------------------------------------------------------------
Author:
	Eduard [esp] Schwan
------------------------------------------------------------------------------
	from Persistence of Vision(tm) Ray Tracer
	Copyright 1996 Persistence of Vision Team
------------------------------------------------------------------------------
	NOTICE: This source code file is provided so that users may experiment
	with enhancements to POV-Ray and to port the software to platforms other 
	than those supported by the POV-Ray Team.  There are strict rules under
	which you are permitted to use this file.  The rules are in the file
	named POVLEGAL.DOC which should be distributed with this file. If 
	POVLEGAL.DOC is not available or for more info please contact the POV-Ray
	Team Coordinator by leaving a message in CompuServe's Graphics Developer's
	Forum.  The latest version of POV-Ray may be found there as well.

	This program is based on the popular DKB raytracer version 2.12.
	DKBTrace was originally written by David K. Buck.
	DKBTrace Ver 2.0-2.12 were written by David K. Buck & Aaron A. Collins.
------------------------------------------------------------------------------
Change History:
==============================================================================*/

#ifndef LNKLST_H
#define LNKLST_H

typedef unsigned short LLIndex_t;

// An Element struct, holds link refs and ptr to actual data
typedef struct
{
	void*		fNext;
	void*		fPrev;
	void*		fData;
} LLElRec, *LLElPtr, **LLElHdl;

// A Header struct, holds info for one linked list
typedef struct
{
	short		fMaxElements;
	short		fNumElements;
	LLElPtr		fFirstElement;
	LLElPtr		fLastElement;
} LLHeadRec, *LLHeadPtr, **LLHeadHdl;

// NOTE: Index is 1..n

// House keeping

LLHeadPtr LLCreateList(short MaxElements);

void LLDestroyList(LLHeadPtr LL);

void LLDeleteAllElements(LLHeadPtr LL);

// Add

void LLAddElementByIndex(LLHeadPtr LL, void * DataPtr, short Index);
void LLAppendElement(LLHeadPtr LL, void * DataPtr);

// Delete

void LLDeleteElementByIndex(LLHeadPtr LL, short Index);

// Gets

short LLGetMaxElements(LLHeadPtr LL);

short LLGetNumElements(LLHeadPtr LL);

void * LLGetElementByIndex(LLHeadPtr LL, short Index);

#ifdef LLTEST
void LLTest(void);
#endif

#endif
