/*==============================================================================
Project:	POV

Version:	3

File:	MemAlloc.h

Description:
Routines to handle the Mac-specific memory handling.
------------------------------------------------------------------------------
Author:
	Eduard [esp] Schwan
------------------------------------------------------------------------------
	from Persistence of Vision(tm) Ray Tracer
	Copyright 1996 Persistence of Vision Team
------------------------------------------------------------------------------
	NOTICE: This source code file is provided so that users may experiment
	with enhancements to POV-Ray and to port the software to platforms other 
	than those supported by the POV-Ray Team.  There are strict rules under
	which you are permitted to use this file.  The rules are in the file
	named POVLEGAL.DOC which should be distributed with this file. If 
	POVLEGAL.DOC is not available or for more info please contact the POV-Ray
	Team Coordinator by leaving a message in CompuServe's Graphics Developer's
	Forum.  The latest version of POV-Ray may be found there as well.

	This program is based on the popular DKB raytracer version 2.12.
	DKBTrace was originally written by David K. Buck.
	DKBTrace Ver 2.0-2.12 were written by David K. Buck & Aaron A. Collins.
------------------------------------------------------------------------------
Change History:
==============================================================================*/

#if !defined(MEMALLOC_H)
#define MEMALLOC_H

#include "config.h"

#include <types.h>	// Boolean
#include <stdlib.h>

// rename POV-Ray's MEM.C library calls to call Mac-POV-Ray's memalloc.c routines
#define MALLOC				Mac_malloc
#define CALLOC				Mac_calloc
#define REALLOC				Mac_realloc
#define FREE				Mac_free

// rename Std C's Stdlib library calls to call Mac-POV-Ray's memalloc.c routines
#define malloc(size)		Mac_malloc(size)
#define calloc(nitems,size)	Mac_calloc((nitems),(size))
#define realloc(ptr,size)	Mac_realloc((ptr),(size))
#define free(ptr)			Mac_free(ptr)

void UseTempMem(int doIt);
void AllocateSafetyBuffer(size_t reserve_size);
void PurgeSafetyBuffer(void);
Boolean InLowMemoryMode(void);
void *Mac_malloc(size_t size);
void *Mac_calloc(size_t nmemb, size_t size);
void *Mac_realloc(void * p, size_t newsize);
void Mac_free(void *p);


#endif // MEMALLOC_H
