# 
# ==============================================================================
# Project:	POV-Ray
#
# Version:	3
# 
# File:		POV-Ray.make
# 
# Contents:
# 	Apple MPW Make file, used to generate the POV-Ray application.
#	Now works with MPW 3.1, 3.2, 3.3.1.
# 
# Related Files:
# 	POV.c: Main Program source
# 	POV.rsrc: what rsrcs get appended..
# ------------------------------------------------------------------------------
# Author:
#	Eduard [esp] Schwan
# ------------------------------------------------------------------------------
#	from Persistence of Vision(tm) Ray Tracer
#	Copyright 1996 Persistence of Vision Team
#------------------------------------------------------------------------------
#	NOTICE: This source code file is provided so that users may experiment
#	with enhancements to POV-Ray and to port the software to platforms other 
#	than those supported by the POV-Ray Team.  There are strict rules under
#	which you are permitted to use this file.  The rules are in the file
#	named POVLEGAL.DOC which should be distributed with this file. If 
#	POVLEGAL.DOC is not available or for more info please contact the POV-Ray
#	Team Coordinator by leaving a message in CompuServe's Graphics Developer's
#	Forum.  The latest version of POV-Ray may be found there as well.
#
#	This program is based on the popular DKB raytracer version 2.12.
#	DKBTrace was originally written by David K. Buck.
#	DKBTrace Ver 2.0-2.12 were written by David K. Buck & Aaron A. Collins.
#------------------------------------------------------------------------------
#More Info:
#	This Macintosh version of POV-Ray was updated and compiled by Eduard Schwan
#	and now builds under Apple MPW 3.3.x, Metrowerks CodeWarrior CW6+, and
#	Symantec C 7, for the 68K and Power Macintosh.
#	The Macintosh code was based (loosely) on the original DKB 0.5 port by
#	Thomas Okken and David Lichtman, with some help from Glenn Sugden.
#	Jim Nitchals was instrumental in getting 1.0 out, Jim and Eduard jointly
#	pulled off 2.x with help from Dave Harr for initial Balloon help and the
#	custom palette code, and 3.0 was done by Eduard Schwan.
#
#	For bug reports regarding the Macintosh version, you may contact:
#	Eduard [esp] Schwan -- esp Software
#		CompuServe: 71513,2161
#		Internet: 71513.2161@compuserve.com
# ------------------------------------------------------------------------------
# Change History:
#	920308	[esp]	Created for POV-Ray
#	920315	[esp]	Added -b compile option
#	920330	[esp]	Did the file dependencies for real (by hand!)
#	920415	[esp]	Added QuickTime and StdCompression files
#	920429	[esp]	Changed QT & SC file placement, and changed app name to POV-Ray
#	920718	[esp]	Updated to 3.2 only
#	920902	[esp]	Tested under ETO#8, MPW 3.2.3, added TemplateMenu.c
#	920923	[esp]	Collected generic routines into a POVLib.o library for others to use
#	921202	[esp]	Removed -mf option from C, removed -FAR from C & Link
#	930425	[esp]	Broke image window stuff out to ImageWindow.c/h
#	930522	[esp]	Changed app signature from PVtm(1.0) to PvRy(2.0)
#	930605	[esp]	Changed srcpath to incpath on all .h files
#	930829	[esp]	Added the split resource files included
#	931001	[esp]	version 2.0 finished (Released on 10/4/93)
#	940204	[esp]	version 2.2 finished (altered mem alloc defines)
#	950108	[esp]	version 3 alpha work, starting adding precompiled headers
#	950917	[esp]	version 3 beta work, zlib/png added
#	960226	[esp]	version 3 beta work, final cleanup
# ==============================================================================
# 

#------------------------------------------------------------------------------
# AppName sets the main application name for the make.
# AppSignature sets the Creator signature for the application.
#------------------------------------------------------------------------------

AppName			=	POV-Ray
AppSignature	=	'POV3'


#------------------------------------------------------------------------------
# Use variables for the compiler/linker tools
#------------------------------------------------------------------------------

C				= C			# SC (for MPW #19 or later...)
Rez				= Rez
Lib				= Lib
Link			= Link

#------------------------------------------------------------------------------
# These are the paths to prefix source and object files, & additional folders
#------------------------------------------------------------------------------
macpath			=	:MacSource:
corepath		=	:SOURCE:
zlibpath		=	{corepath}zlib:
pngpath			=	{corepath}libpng:
rezpath			=	{macpath}
incpath			=	{macpath}
objpath			= 	:mpwobj:

#------------------------------------------------------------------------------
# Options for our compilers:
#	-sym on: tells the compilers and linker to emit symbol information for
#		a source level debugger, such as SADE.
#	-i {ExtraIncludes}: means to look for any #include files in the specified
#		directory, as well as the normal set.
#	-r: tells the C compiler to require function prototypes.
#	-mbg off: tells the compilers to not emit low-level debugger names. This
#		saves on file space, but you may wish to remove this option if you
#		need to debug with something like Macsbug.
#	-rd: for Rez means to suppress warnings for redeclared types (we redeclare
#		'RECT' because its not included in MPW 3.0).
#	-append: means to add the resources to the target file, rather than
#		deleting all the ones that are there first.
#	-d Signature...: is a way of passing our application's signature to Rez.
#		With this mechanism, we can define our signature here, and export
#		it to Rez, so that we don't have to declare it there, too.
#	-sn STDCLIB=Main: puts all the routines that would normally go into the
#		STDCLIB segment into the Main segment. This is done so that when we
#		call upon any low-level utilities, we don't potentially move memory
#		by loading in a segment.
#------------------------------------------------------------------------------

DebugStmts			=	#-d NEEDS_DEBUG					# turn on to add internal debug stmts
CAltOptions			=	-r -b2							# warn on no prototypes, put stringconsts in code segs
Cmp020Options		=	-d NEEDS_68020 -mc68020			# turn on to force 68020 or better CPU code
CmpColorQD			=	-d NEEDS_COLORQD				# turn on to force Color Quickdraw ROM Support
Cmp32BitQD			=	-d NEEDS_32BITQD				# turn on to force 32Bit Quickdraw ROM Support
CmpFPUOptions		=	-d NEEDS_FPU -mc68881 -elems881 # turn on to force 68881 FPU calls
NEEDS_PROF			=	#-d NEEDS_PROF					# turn on to do profiling
LoadDumpOptions		=	#-k {objpath}					# turn on to do load/dump files
# Far model support (fardata is needed always, farcode for non-fpu version)
CModel				=	-model fardata # farcode
LinkModel			=	-model far

PerfCmpOptions		=	#-d NEEDS_PERF=1 -trace on		# turn on to enable performance code
PerfLinkOptions		=	#"{Libraries}"Proff.o -sn ProffSeg=Main	# turn on to use performance tools

# non-FPU libs
#LinkMathLibs			=	"{CLibraries}"CSANELib.o "{CLibraries}"Math.o
# FPU libs
LinkMathLibs		=	"{CLibraries}"Clib881.o "{CLibraries}"CSANELib881.o "{CLibraries}"Math881.o

MapOptions			=	-l > {AppName}.Map			# use this for human-readable map
SymOptions			=	-sym full					# turn on to debug with SADE
MbgOptions			=	-mbg on						# turn on to insert MacsBug labels

MacFolders			=	-i {incpath}
ZLibFolders			=	-i {zlibpath}
PNGFolders			=	-i {pngpath}
CoreFolders			=	-i {corepath}
RInc2Folders		=	#-i {rezpath}
ZLIB_Opts			=	-d HAVE_MEMCPY -d MY_ZCALLOC -d STDC -d MACOS {ZLibFolders}
PNG_Opts			=	{ZLIB_Opts} {PNGFolders}
CInc2Folders		=	{MacFolders} {CoreFolders} {ZLIB_Opts} {PNG_Opts}


MyCOptions		=	
	{COptions} {CModel} {SymOptions} {LoadDumpOptions} 
	{MemFlags} {DebugStmts} {CUSTOM_VERSION} {MbgOptions} 
	{Cmp020Options} {CmpFPUOptions} {CmpColorQD} {Cmp32bitQD} 
	 -d SystemSixOrLater=1 -d SystemSevenOrLater=1 -d OLDROUTINENAMES=false -d OLDROUTINELOCATIONS=false

RezOptions		=	
	-rd -append {RInc2Folders} 
	-d AppSignature={AppSignature} 
	-d AppName={AppName}


#------------------------------------------------------------------------------

ShowProgress		=  Echo "# `date -s` ----- Building {Targ}"


#------------------------------------------------------------------------------
# These are the include file hierarchy shorthand variables for POVLib
#------------------------------------------------------------------------------

# The POV Library file name
POVLib			= POVLib

# The headers used

h_AeAppHdlr = 
			{incpath}AeAppHdlr.h
h_ArgvTrix = 
			{incpath}ArgvTrix.h
h_DialogUtils = 
			{incpath}DialogUtils.h
h_FileQueue = 
			{incpath}FileQueue.h
h_LnkLst = 
			{incpath}LnkLst.h
h_CursorUtils = 
			{incpath}CursorUtils.h
h_MemAlloc = 
			{incpath}MemAlloc.h
h_stdiop2w = 
			{incpath}Stdio_p2w.h
h_Printf2Window = 
			{h_stdiop2w} 
			{incpath}Printf2Window.h
h_ProgressDialog = 
			{incpath}ProgressDialog.h
h_ScreenUtils = 
			{incpath}ScreenUtils.h
h_VolsPaths = 
			{incpath}VolsPaths.h
h_POVLib = 
			{h_ArgvTrix} 
			{h_DialogUtils} 
			{h_FileQueue} 
			{h_LnkLst} 
			{h_CursorUtils} 
			{h_MemAlloc} 
			{h_Printf2Window} 
			{h_ProgressDialog} 
			{h_ScreenUtils} 
			{h_VolsPaths}

# the objects used
o_AeAppHdlr = 
			{objpath}AeAppHdlr.o
o_ArgvTrix = 
			{objpath}ArgvTrix.o
o_DialogUtils = 
			{objpath}DialogUtils.o
o_FileQueue = 
			{objpath}FileQueue.o
o_LnkLst = 
			{objpath}LnkLst.o
o_CursorUtils = 
			{objpath}CursorUtils.o
o_MemAlloc = 
			{objpath}MemAlloc.o
o_Printf2Window = 
			{objpath}Printf2Window.o
o_ProgressDialog = 
			{objpath}ProgressDialog.o
o_ScreenUtils = 
			{objpath}ScreenUtils.o
o_VolsPaths = 
			{objpath}VolsPaths.o

o_POVLibs = 
			{o_ArgvTrix} 
			{o_DialogUtils} 
			{o_FileQueue} 
			{o_LnkLst} 
			{o_CursorUtils} 
			{o_MemAlloc} 
			{o_Printf2Window} 
			{o_ProgressDialog} 
			{o_ScreenUtils} 
			{o_VolsPaths}


#------------------------------------------------------------------------------
# These are the include file hierarchy shorthand variables for the Application
#------------------------------------------------------------------------------

h_TextEditor = 
			{incpath}TextEditor.h

h_ImageWindow = 
		{incpath}ImageWindow.h

h_TemplateMenu = 
		{incpath}TemplateMenu.h

h_config = 
		{incpath}Config.h 
		{h_POVLib}

h_povmac = 
		{incpath}PovMac.h 
		{incpath}Animate.h 
		{h_POVLib}

#------------------------------------------------------------------------------
# These are the compiled resource files that we want to link with.
# If any one of these changes, then we invoke the Rez command.
#------------------------------------------------------------------------------

rsrcFiles = 
		{rezpath}POV.rsrc

#------------------------------------------------------------------------------
# These are the objects that we want to link with. If any one of these
# changes, then we invoke the Link command.
#------------------------------------------------------------------------------

MacObjects = 
		{objpath}AboutBox.c.o 
		{objpath}AEUtils.c.o 
		{objpath}Animate.c.o 
		{objpath}AppPrefs.c.o 
		{objpath}FilePrefs.c.o 
		{objpath}ImageWindow.c.o 
		{objpath}Pov.c.o 
		{objpath}SaveCmpPict.c.o 
		{objpath}SplashScreen.c.o 
		{objpath}StdFolder.c.o 
		{objpath}TemplateMenu.c.o 
		{objpath}TextEditor.C.o 
		{objpath}{POVLib}.o

		#{libraries}StdCompressionGlue.o	# QuickTime StdCmp Dialog
		# Note that in MPW 3.3 and later, StdCompressionGlue.o is in {libraries}

ZLibObjects = 
		{objpath}ADLER32.C.o 
		{objpath}COMPRESS.C.o 
		{objpath}CRC32.C.o 
		{objpath}DEFLATE.C.o 
		{objpath}GZIO.C.o 
		{objpath}INFBLOCK.C.o 
		{objpath}INFCODES.C.o 
		{objpath}INFFAST.C.o 
		{objpath}INFLATE.C.o 
		{objpath}INFTREES.C.o 
		{objpath}INFUTIL.C.o 
		{objpath}TREES.C.o 
		{objpath}UNCOMPR.C.o 
		{objpath}ZUTIL.C.o 
		{objpath}ZSupport.c.o 

PNGObjects = 
		{objpath}PNG.C.o 
		{objpath}PNGERROR.C.o 
		{objpath}PNGMEM.C.o 
		{objpath}PNGRCB.C.o 
		{objpath}PNGREAD.C.o 
		{objpath}PNGRTRAN.C.o 
		{objpath}PNGRUTIL.C.o 
		{objpath}PNGTRANS.C.o 
		{objpath}PNGWRITE.C.o 
		{objpath}PNGWTRAN.C.o 
		{objpath}PNGWUTIL.C.o 


POVCoreObjects = 
		{objpath}ATMOSPH.C.o	
		{objpath}BBOX.C.o		
		{objpath}BEZIER.C.o		
		{objpath}BLOB.C.o		
		{objpath}BOXES.C.o		
		{objpath}BSPHERE.C.o	
		{objpath}CAMERA.C.o		
		{objpath}CHI2.C.o		
		{objpath}COLOUR.C.o		
		{objpath}CONES.C.o		
		{objpath}CSG.C.o		
		{objpath}DISCS.C.o		
		{objpath}EXPRESS.C.o	
		{objpath}FRACTAL.C.o	
		{objpath}GIF.C.o		
		{objpath}GIFDECOD.C.o	
		{objpath}HALOS.C.o		
		{objpath}HCMPLX.C.o		
		{objpath}HFIELD.C.o		
		{objpath}IFF.C.o		
		{objpath}IMAGE.C.o		
		{objpath}LATHE.C.o		
		{objpath}LBUFFER.C.o	
		{objpath}LIGHTING.C.o	
		{objpath}MATRICES.C.o	
		{objpath}MEM.C.o		
		{objpath}MESH.C.o		
		{objpath}NORMAL.C.o		
		{objpath}OBJECTS.C.o	
		{objpath}OCTREE.C.o		
		{objpath}OPTIN.C.o		
		{objpath}OPTOUT.C.o		
		{objpath}PARSE.C.o		
		{objpath}PARSTXTR.C.o	
		{objpath}PATTERN.C.o	
		{objpath}PGM.C.o		
		{objpath}PIGMENT.C.o	
		{objpath}PLANES.C.o		
		{objpath}PNG_POV.C.o	
		{objpath}POINT.C.o		
		{objpath}POLY.C.o		
		{objpath}POLYGON.C.o	
		{objpath}POLYSOLV.C.o	
		{objpath}POVRAY.C.o		
		{objpath}PPM.C.o		
		{objpath}PRISM.C.o		
		{objpath}QUADRICS.C.o	
		{objpath}QUATERN.C.o	
		{objpath}RADIOSIT.C.o	
		{objpath}RAD_DATA.C.o	
		{objpath}RAY.C.o		
		{objpath}RENDER.C.o		
		{objpath}SOR.C.o		
		{objpath}SPHERES.C.o	
		{objpath}SUPER.C.o		
		{objpath}TARGA.C.o		
		{objpath}TEXTURE.C.o	
		{objpath}TOKENIZE.C.o	
		{objpath}TORUS.C.o		
		{objpath}TRIANGLE.C.o	
		{objpath}TRUETYPE.C.o	
		{objpath}TXTTEST.C.o	
		{objpath}USERIO.C.o		
		{objpath}VBUFFER.C.o	
		{objpath}VLBUFFER.C.o	
		{objpath}WARPS.C.o		

AppObjects = 
		{MacObjects} 
		{ZLibObjects} 
		{PNGObjects} 
		{POVCoreObjects}


#------------------------------------------------------------------------------
# precompiled header file thingies
#------------------------------------------------------------------------------

PCHSrcFile = 
			{srcpath}POVMacHeaders.pch
PCHOutFile = 
			{objpath}POVMPW68K.dump


#----------------
# Good ol' MPW 3.2 std libraries are now in their own segments, for reasons known only to
# the MPW team..  Put them back into the main segment so we don't get totally fragmented..
SegmentMappings	=	
	-sn INTENV=Main2 
	-sn PASLIB=Main2 
	-sn SADEV=Main2 
	-sn STDCLIB=Main2 
	-sn STDIO=Main2 
	-sn CSANELib=Main2 
	-sn SANELIB=Main2

#------------------------------------------------------------------------------
# -d means: suppress dup symbols
LinkOptions		=	
	-d {LinkModel} {SymOptions} {PerfLinkOptions} {SegmentMappings} {MapOptions}

CLibs			=	
	{LinkMathLibs} 
	"{CLibraries}"StdCLib.o 
	"{Libraries}Runtime.o" 
	"{Libraries}Interface.o"

#------------------------------------------------------------------------------
# Build rule that links our application together. If any of our objects 
# change, or this makefile changes, then we relink.  The dummy prerequisite
# ShellForce must come before any other prerequisites for {AppName}
#------------------------------------------------------------------------------

{AppName}			{AppName}.make  {AppObjects}
	{ShowProgress} ; Echo "# Linking..."
	{Link} {LinkOptions} -o {Targ} {AppObjects} {CLibs}
	# Set file's creator type, turn on bundle bit, turn off inited bit
	SetFile {Targ} -t APPL -c {AppSignature} -a Bi


#------------------------------------------------------------------------------
# Build rule that creates our resources and adds them to the application
#------------------------------------------------------------------------------

{AppName}			{AppName}.make  {rsrcFiles}
	{ShowProgress} ; Echo "# Rezzing..."
	Echo "include "POV.rsrc";" | {Rez} {RezOptions} -o {Targ}


#------------------------------------------------------------------------------
# Segment Names for the code segments
#------------------------------------------------------------------------------

segAnimate		= -s 'povAnim'
segBezier		= -s 'povBezier'
segBlob			= -s 'povBlob'
segBound		= -s 'povBound'
segBoxes		= -s 'povBoxes'
segCamera		= -s 'povCamera'
segCones		= -s 'povCones'
segCSG			= -s 'povCSG'
segDiscs		= -s 'povDiscs'
segHF			= -s 'povHF'
segJulia		= -s 'povJulia'
segImage		= -s 'povImage'
segLighting		= -s 'povLighting'
segLBuffer		= -s 'povLBuffer'
segNormal		= -s 'povNormal'
segParse		= -s 'povParse'
segParse2		= -s 'povParse2'
segPigment		= -s 'povPigment'
segPix			= -s 'povPix'
segPlanes		= -s 'povPlanes'
segPoly			= -s 'povPoly'
segPolygon		= -s 'povPolygon'
segPOVLib		= -s 'povLib'
segPOV			= -s 'povMac'
segPOVRay		= -s 'povRay'
segPrefs		= -s 'segPrefs'
segPrim1		= -s 'povPrim1'
segPrim2		= -s 'povPrim2'
segRender		= -s 'povRender'
segTex			= -s 'povTex'
segTokenize		= -s 'povToke'
segTri			= -s 'povTri'
segTT			= -s 'povTrueType'
segSphere		= -s 'povSphere'
segWindows		= -s 'povWindows'
segColour		= -s 'povColour'
segTarga		= -s 'povTarga'
segTemplate		= -s 'povTemplate'

segZlib			= -s 'povZLib'
segPNG			= -s 'povPNG'


#------------------------------------------------------------------------------
# Build rules that create the prcompiled header file
#------------------------------------------------------------------------------

#{PCHOutFile}				{PCHSrcFile}
#	{ShowProgress}
#	Echo "# `date -s` ----- (Pre-compiling)"
#	{C} {PCHSrcFile} -k {objpath} {MyCOptions} {CAltOptions}


#------------------------------------------------------------------------------
# Build rules that compile our POV Library from its source files
#------------------------------------------------------------------------------

{o_AeAppHdlr}				{h_POVLib} {macpath}AeAppHdlr.c
	{ShowProgress}
	{C} {macpath}AeAppHdlr.c {segPOVLib} {MyCOptions} {CAltOptions} {CInc2Folders} -o {o_AeAppHdlr}

{o_ArgvTrix}				{h_POVLib} {macpath}ArgvTrix.c
	{ShowProgress}
	{C} {macpath}ArgvTrix.c {segPOVLib} {MyCOptions} {CAltOptions} {CInc2Folders} -o {o_ArgvTrix}

{o_DialogUtils}				{h_POVLib} {macpath}DialogUtils.c
	{ShowProgress}
	{C} {macpath}DialogUtils.c {segPOVLib} {MyCOptions} {CAltOptions} {CInc2Folders} -o {o_DialogUtils}

{o_FileQueue}				{h_POVLib} {macpath}FileQueue.c
	{ShowProgress}
	{C} {macpath}FileQueue.c {segPOVLib} {MyCOptions} {CAltOptions} {CInc2Folders} -o {o_FileQueue}

{o_LnkLst}				{h_POVLib} {macpath}LnkLst.c
	{ShowProgress}
	{C} {macpath}LnkLst.c {segPOVLib} {MyCOptions} {CAltOptions} {CInc2Folders} -o {o_LnkLst}

{o_CursorUtils}			  {h_POVLib} {macpath}CursorUtils.c
	{ShowProgress}
	{C} {macpath}CursorUtils.c {segPOVLib} {MyCOptions} {CAltOptions} {CInc2Folders} -o {o_CursorUtils}

{o_MemAlloc}			  {h_POVLib} {macpath}MemAlloc.c
	{ShowProgress}
	{C} {macpath}MemAlloc.c {segPOVLib} {MyCOptions} {CAltOptions} {CInc2Folders} -o {o_MemAlloc}

{o_Printf2Window}			{h_POVLib}  {macpath}Printf2Window.c
	{ShowProgress}
	{C} {macpath}Printf2Window.c {segPOVLib} {MyCOptions} {CAltOptions} {CInc2Folders} -o {o_Printf2Window}

{o_ProgressDialog}			{h_POVLib}  {macpath}ProgressDialog.c
	{ShowProgress}
	{C} {macpath}ProgressDialog.c {segPOVLib} {MyCOptions} {CAltOptions} {CInc2Folders} -o {o_ProgressDialog}

{o_ScreenUtils}				{h_POVLib}  {macpath}ScreenUtils.c
	{ShowProgress}
	{C} {macpath}ScreenUtils.c {segPOVLib} {MyCOptions} {CAltOptions} {CInc2Folders} -o {o_ScreenUtils}

{o_VolsPaths}				{h_POVLib}  {macpath}VolsPaths.c
	{ShowProgress}
	{C} {macpath}VolsPaths.c {segPOVLib} {MyCOptions} {CAltOptions} {CInc2Folders} -o {o_VolsPaths}

{objpath}{POVLib}.o			{o_POVLibs}
	{ShowProgress}
	{Lib} {o_POVLibs} -o {objpath}{POVLib}.o


#------------------------------------------------------------------------------
# Build rules that compile our Mac-specific source files
#------------------------------------------------------------------------------

{objpath}AboutBox.c.o			{h_POVLib} {incpath}AboutBox.h {macpath}AboutBox.c
	{ShowProgress}
	{C} {macpath}AboutBox.c {segPOV} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}AboutBox.c.o

{objpath}AEUtils.c.o			{h_POVLib} {incpath}AEUtils.h {macpath}AEUtils.c
	{ShowProgress}
	{C} {macpath}AEUtils.c {segAnimate} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}AEUtils.c.o

{objpath}Animate.c.o			{h_POVLib} {incpath}Animate.h {macpath}Animate.c
	{ShowProgress}
	{C} {macpath}Animate.c {segAnimate} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}Animate.c.o

{objpath}AppPrefs.c.o			{h_POVLib} {incpath}AppPrefs.h {macpath}AppPrefs.c
	{ShowProgress}
	{C} {macpath}AppPrefs.c {segPrefs} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}AppPrefs.c.o

{objpath}FilePrefs.c.o			{h_POVLib} {incpath}FilePrefs.h {macpath}FilePrefs.c
	{ShowProgress}
	{C} {macpath}FilePrefs.c {segPrefs} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}FilePrefs.c.o

{objpath}ImageWindow.c.o		{macpath}ImageWindow.c {h_ImageWindow}
	{ShowProgress}
	{C} {macpath}ImageWindow.c {segWindows} {MyCOptions} {CAltOptions} {CInc2Folders} -o {objpath}ImageWindow.c.o

{objpath}Pov.C.o				{h_ImageWindow} {h_TextEditor} {h_povmac} {h_POVLib} {h_TemplateMenu} {incpath}SaveCmpPict.h {macpath}Pov.C
	{ShowProgress}
	{C} {macpath}Pov.C {segPOV} {MyCOptions} {CAltOptions} {CInc2Folders} -o {objpath}Pov.C.o

{objpath}SaveCmpPict.c.o		{incpath}SaveCmpPict.h {macpath}SaveCmpPict.c
	{ShowProgress}
	{C} {macpath}SaveCmpPict.c {segPix} {MyCOptions} {CAltOptions} {CInc2Folders} -o {objpath}SaveCmpPict.c.o

{objpath}SplashScreen.c.o			{h_POVLib} {incpath}SplashScreen.h {macpath}SplashScreen.c
	{ShowProgress}
	{C} {macpath}SplashScreen.c {segPOV} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}SplashScreen.c.o

{objpath}stdfolder.c.o		{incpath}SaveCmpPict.h {macpath}stdfolder.c
	{ShowProgress}
	{C} {macpath}stdfolder.c {segPrefs} {MyCOptions} {CAltOptions} {CInc2Folders} -o {objpath}stdfolder.c.o

{objpath}TemplateMenu.c.o		{h_TemplateMenu}  {h_povmac}  {macpath}TemplateMenu.c
	{ShowProgress}
	{C} {macpath}TemplateMenu.c {segTemplate} {MyCOptions} {CAltOptions} {CInc2Folders} -o {objpath}TemplateMenu.c.o

{objpath}TextEditor.C.o			{h_TextEditor} {h_povmac} {macpath}TextEditor.C
	{ShowProgress}
	{C} {macpath}TextEditor.C {segWindows} {MyCOptions} {CAltOptions} {CInc2Folders} -o {objpath}TextEditor.C.o

{objpath}ZSupport.c.o			{macpath}ZSupport.c
	{ShowProgress}
	{C} {macpath}ZSupport.c {segWindows} {MyCOptions} {CAltOptions} {CInc2Folders} {ZLIB_Opts} -o {objpath}ZSupport.c.o


#------------------------------------------------------------------------------
# Build rules that compile the ZLib source files
#------------------------------------------------------------------------------

{objpath}ADLER32.C.o			{zlibpath}ADLER32.C
	{ShowProgress}
	{C} {zlibpath}ADLER32.C {segWindows} {MyCOptions} {CAltOptions} {ZLIB_Opts} -o {objpath}ADLER32.C.o

{objpath}COMPRESS.C.o			{zlibpath}COMPRESS.C
	{ShowProgress}
	{C} {zlibpath}COMPRESS.C {segWindows} {MyCOptions} {CAltOptions} {ZLIB_Opts} -o {objpath}COMPRESS.C.o

{objpath}CRC32.C.o			{zlibpath}CRC32.C
	{ShowProgress}
	{C} {zlibpath}CRC32.C {segWindows} {MyCOptions} {CAltOptions} {ZLIB_Opts} -o {objpath}CRC32.C.o

{objpath}DEFLATE.C.o			{zlibpath}DEFLATE.C
	{ShowProgress}
	{C} {zlibpath}DEFLATE.C {segWindows} {MyCOptions} {CAltOptions} {ZLIB_Opts} -o {objpath}DEFLATE.C.o

{objpath}GZIO.C.o			{zlibpath}GZIO.C
	{ShowProgress}
	{C} {zlibpath}GZIO.C {segWindows} {MyCOptions} {CAltOptions} {ZLIB_Opts} -o {objpath}GZIO.C.o

{objpath}INFBLOCK.C.o			{zlibpath}INFBLOCK.C
	{ShowProgress}
	{C} {zlibpath}INFBLOCK.C {segWindows} {MyCOptions} {CAltOptions} {ZLIB_Opts} -o {objpath}INFBLOCK.C.o

{objpath}INFCODES.C.o			{zlibpath}INFCODES.C
	{ShowProgress}
	{C} {zlibpath}INFCODES.C {segWindows} {MyCOptions} {CAltOptions} {ZLIB_Opts} -o {objpath}INFCODES.C.o

{objpath}INFFAST.C.o			{zlibpath}INFFAST.C
	{ShowProgress}
	{C} {zlibpath}INFFAST.C {segWindows} {MyCOptions} {CAltOptions} {ZLIB_Opts} -o {objpath}INFFAST.C.o

{objpath}INFLATE.C.o			{zlibpath}INFLATE.C
	{ShowProgress}
	{C} {zlibpath}INFLATE.C {segWindows} {MyCOptions} {CAltOptions} {ZLIB_Opts} -o {objpath}INFLATE.C.o

{objpath}INFTREES.C.o			{zlibpath}INFTREES.C
	{ShowProgress}
	{C} {zlibpath}INFTREES.C {segWindows} {MyCOptions} {CAltOptions} {ZLIB_Opts} -o {objpath}INFTREES.C.o

{objpath}INFUTIL.C.o			{zlibpath}INFUTIL.C
	{ShowProgress}
	{C} {zlibpath}INFUTIL.C {segWindows} {MyCOptions} {CAltOptions} {ZLIB_Opts} -o {objpath}INFUTIL.C.o

{objpath}TREES.C.o			{zlibpath}TREES.C
	{ShowProgress}
	{C} {zlibpath}TREES.C {segWindows} {MyCOptions} {CAltOptions} {ZLIB_Opts} -o {objpath}TREES.C.o

{objpath}UNCOMPR.C.o			{zlibpath}UNCOMPR.C
	{ShowProgress}
	{C} {zlibpath}UNCOMPR.C {segWindows} {MyCOptions} {CAltOptions} {ZLIB_Opts} -o {objpath}UNCOMPR.C.o

{objpath}ZUTIL.C.o			{zlibpath}ZUTIL.C
	{ShowProgress}
	{C} {zlibpath}ZUTIL.C {segWindows} {MyCOptions} {CAltOptions} {ZLIB_Opts} -o {objpath}ZUTIL.C.o


#------------------------------------------------------------------------------
# Build rules that compile the PNG source files
#------------------------------------------------------------------------------

{objpath}PNG.C.o				{pngpath}PNG.C
	{ShowProgress}
	{C} {pngpath}PNG.C {segWindows} {MyCOptions} {CAltOptions} {PNG_Opts} -o {objpath}PNG.C.o

{objpath}PNGERROR.C.o			{pngpath}PNGERROR.C
	{ShowProgress}
	{C} {pngpath}PNGERROR.C {segWindows} {MyCOptions} {CAltOptions} {PNG_Opts} -o {objpath}PNGERROR.C.o

{objpath}PNGMEM.C.o				{pngpath}PNGMEM.C
	{ShowProgress}
	{C} {pngpath}PNGMEM.C {segWindows} {MyCOptions} {CAltOptions} {PNG_Opts} -o {objpath}PNGMEM.C.o

{objpath}PNGRCB.C.o				{pngpath}PNGRCB.C
	{ShowProgress}
	{C} {pngpath}PNGRCB.C {segWindows} {MyCOptions} {CAltOptions} {PNG_Opts} -o {objpath}PNGRCB.C.o

{objpath}PNGREAD.C.o			{pngpath}PNGREAD.C
	{ShowProgress}
	{C} {pngpath}PNGREAD.C {segWindows} {MyCOptions} {CAltOptions} {PNG_Opts} -o {objpath}PNGREAD.C.o

{objpath}PNGRTRAN.C.o			{pngpath}PNGRTRAN.C
	{ShowProgress}
	{C} {pngpath}PNGRTRAN.C {segWindows} {MyCOptions} {CAltOptions} {PNG_Opts} -o {objpath}PNGRTRAN.C.o

{objpath}PNGRUTIL.C.o			{pngpath}PNGRUTIL.C
	{ShowProgress}
	{C} {pngpath}PNGRUTIL.C {segWindows} {MyCOptions} {CAltOptions} {PNG_Opts} -o {objpath}PNGRUTIL.C.o

{objpath}PNGTRANS.C.o			{pngpath}PNGTRANS.C
	{ShowProgress}
	{C} {pngpath}PNGTRANS.C {segWindows} {MyCOptions} {CAltOptions} {PNG_Opts} -o {objpath}PNGTRANS.C.o

{objpath}PNGWRITE.C.o			{pngpath}PNGWRITE.C
	{ShowProgress}
	{C} {pngpath}PNGWRITE.C {segWindows} {MyCOptions} {CAltOptions} {PNG_Opts} -o {objpath}PNGWRITE.C.o

{objpath}PNGWTRAN.C.o			{pngpath}PNGWTRAN.C
	{ShowProgress}
	{C} {pngpath}PNGWTRAN.C {segWindows} {MyCOptions} {CAltOptions} {PNG_Opts} -o {objpath}PNGWTRAN.C.o

{objpath}PNGWUTIL.C.o			{pngpath}PNGWUTIL.C
	{ShowProgress}
	{C} {pngpath}PNGWUTIL.C {segWindows} {MyCOptions} {CAltOptions} {PNG_Opts} -o {objpath}PNGWUTIL.C.o



#------------------------------------------------------------------------------
# Build rules that compile the POV-Ray Core source files
#------------------------------------------------------------------------------

{objpath}ATMOSPH.C.o			 {corepath}ATMOSPH.C
	{ShowProgress}
	{C} {corepath}ATMOSPH.C {segLighting} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}ATMOSPH.C.o

{objpath}BBOX.C.o			 {corepath}BBOX.C
	{ShowProgress}
	{C} {corepath}BBOX.C {segBound} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}BBOX.C.o

{objpath}BEZIER.C.o			 {corepath}BEZIER.C
	{ShowProgress}
	{C} {corepath}BEZIER.C {segBezier} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}BEZIER.C.o

{objpath}BLOB.C.o			 {corepath}BLOB.C
	{ShowProgress}
	{C} {corepath}BLOB.C {segBlob} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}BLOB.C.o

{objpath}BOXES.C.o			 {corepath}BOXES.C
	{ShowProgress}
	{C} {corepath}BOXES.C {segBoxes} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}BOXES.C.o

{objpath}BSPHERE.C.o			 {corepath}BSPHERE.C
	{ShowProgress}
	{C} {corepath}BSPHERE.C {segBound} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}BSPHERE.C.o

{objpath}CAMERA.C.o			 {corepath}CAMERA.C
	{ShowProgress}
	{C} {corepath}CAMERA.C {segCamera} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}CAMERA.C.o

{objpath}CHI2.C.o			 {corepath}CHI2.C
	{ShowProgress}
	{C} {corepath}CHI2.C {segBound} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}CHI2.C.o

{objpath}COLOUR.C.o			 {corepath}COLOUR.C
	{ShowProgress}
	{C} {corepath}COLOUR.C {segColour} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}COLOUR.C.o

{objpath}CONES.C.o			 {corepath}CONES.C
	{ShowProgress}
	{C} {corepath}CONES.C {segCones} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}CONES.C.o

{objpath}CSG.C.o			 {corepath}CSG.C
	{ShowProgress}
	{C} {corepath}CSG.C {segCSG} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}CSG.C.o

{objpath}DISCS.C.o			 {corepath}DISCS.C
	{ShowProgress}
	{C} {corepath}DISCS.C {segDiscs} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}DISCS.C.o

{objpath}EXPRESS.C.o			{corepath}EXPRESS.C
	{ShowProgress}
	{C} {corepath}EXPRESS.C {segParse2} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}EXPRESS.C.o

{objpath}FRACTAL.C.o			{corepath}FRACTAL.C
	{ShowProgress}
	{C} {corepath}FRACTAL.C {segJulia} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}FRACTAL.C.o

{objpath}GIF.C.o			{corepath}GIF.C
	{ShowProgress}
	{C} {corepath}GIF.C {segPix} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}GIF.C.o

{objpath}GIFDECOD.C.o		{corepath}GIFDECOD.C
	{ShowProgress}
	{C} {corepath}GIFDECOD.C {segPix} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}GIFDECOD.C.o

{objpath}HALOS.C.o			 {corepath}HALOS.C
	{ShowProgress}
	{C} {corepath}HALOS.C {segHF} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}HALOS.C.o

{objpath}HCMPLX.C.o			 {corepath}HCMPLX.C
	{ShowProgress}
	{C} {corepath}HCMPLX.C {segJulia} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}HCMPLX.C.o

{objpath}HFIELD.C.o			 {corepath}HFIELD.C
	{ShowProgress}
	{C} {corepath}HFIELD.C {segHF} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}HFIELD.C.o

{objpath}IFF.C.o			{corepath}IFF.C
	{ShowProgress}
	{C} {corepath}IFF.C {segPix} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}IFF.C.o

{objpath}IMAGE.C.o			{corepath}IMAGE.C
	{ShowProgress}
	{C} {corepath}IMAGE.C {segImage} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}IMAGE.C.o

{objpath}LATHE.C.o		 {corepath}LATHE.C
	{ShowProgress}
	{C} {corepath}LATHE.C {segPolygon} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}LATHE.C.o

{objpath}LBUFFER.C.o		 {corepath}LBUFFER.C
	{ShowProgress}
	{C} {corepath}LBUFFER.C {segLBuffer} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}LBUFFER.C.o

{objpath}LIGHTING.C.o		 {corepath}LIGHTING.C
	{ShowProgress}
	{C} {corepath}LIGHTING.C {segLighting} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}LIGHTING.C.o

{objpath}MATRICES.C.o		 {corepath}MATRICES.C
	{ShowProgress}
	{C} {corepath}MATRICES.C {segPrim2} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}MATRICES.C.o

{objpath}MEM.C.o		 {corepath}MEM.C
	{ShowProgress}
	{C} {corepath}MEM.C {segPOVLib} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}MEM.C.o

{objpath}MESH.C.o		 {corepath}MESH.C
	{ShowProgress}
	{C} {corepath}MESH.C {segTri} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}MESH.C.o

{objpath}NORMAL.C.o		 {corepath}NORMAL.C
	{ShowProgress}
	{C} {corepath}NORMAL.C {segNormal} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}NORMAL.C.o

{objpath}OCTREE.C.o		 {corepath}OCTREE.C
	{ShowProgress}
	{C} {corepath}OCTREE.C {segLighting} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}OCTREE.C.o

{objpath}OBJECTS.C.o		 {corepath}OBJECTS.C
	{ShowProgress}
	{C} {corepath}OBJECTS.C {segPrim1} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}OBJECTS.C.o

{objpath}OPTIN.C.o			 {corepath}OPTIN.C {corepath}OPTIN.H
	{ShowProgress}
	{C} {corepath}OPTIN.C {segParse} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}OPTIN.C.o

{objpath}OPTOUT.C.o			 {corepath}OPTOUT.C {corepath}OPTOUT.H
	{ShowProgress}
	{C} {corepath}OPTOUT.C {segParse} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}OPTOUT.C.o

{objpath}PARSE.C.o			 {corepath}PARSE.C {corepath}PARSE.H
	{ShowProgress}
	{C} {corepath}PARSE.C {segParse} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}PARSE.C.o

{objpath}PARSTXTR.C.o			 {corepath}PARSTXTR.C {corepath}PARSTXTR.H
	{ShowProgress}
	{C} {corepath}PARSTXTR.C {segParse2} {MyCOptions} {CAltOptions} {CInc2Folders} {PNG_Opts} {PerfCmpOptions} -o {objpath}PARSTXTR.C.o

{objpath}PATTERN.C.o			 {corepath}PATTERN.C {corepath}PATTERN.H
	{ShowProgress}
	{C} {corepath}PATTERN.C {segTex} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}PATTERN.C.o

{objpath}PGM.C.o			{corepath}PGM.C
	{ShowProgress}
	{C} {corepath}PGM.C {segPix} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}PGM.C.o

{objpath}PIGMENT.C.o			 {corepath}PIGMENT.C
	{ShowProgress}
	{C} {corepath}PIGMENT.C {segPigment} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}PIGMENT.C.o

{objpath}PLANES.C.o			 {corepath}PLANES.C
	{ShowProgress}
	{C} {corepath}PLANES.C {segPlanes} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}PLANES.C.o

{objpath}PNG_POV.C.o			{corepath}PNG_POV.C
	{ShowProgress}
	{C} {corepath}PNG_POV.C {segPix} {MyCOptions} {CAltOptions} {PNG_Opts} {CInc2Folders} {PNG_Opts} {PerfCmpOptions} -o {objpath}PNG_POV.C.o

{objpath}POINT.C.o			 {corepath}POINT.C
	{ShowProgress}
	{C} {corepath}POINT.C {segPrim1} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}POINT.C.o

{objpath}POLY.C.o			 {corepath}POLY.C
	{ShowProgress}
	{C} {corepath}POLY.C {segPoly} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}POLY.C.o

{objpath}POLYGON.C.o			 {corepath}POLYGON.C
	{ShowProgress}
	{C} {corepath}POLYGON.C {segPolygon} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}POLYGON.C.o

{objpath}POLYSOLV.C.o			 {corepath}POLYSOLV.C
	{ShowProgress}
	{C} {corepath}POLYSOLV.C {segPoly} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}POLYSOLV.C.o

{objpath}POVRAY.C.o			 {corepath}POVRAY.C
	{ShowProgress}
	{C} {corepath}POVRAY.C {segPOVRay} {MyCOptions} {CAltOptions} {CInc2Folders} {PNG_Opts} {PerfCmpOptions} -o {objpath}POVRAY.C.o

{objpath}PPM.C.o			{corepath}PPM.C
	{ShowProgress}
	{C} {corepath}PPM.C {segPix} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}PPM.C.o

{objpath}PRISM.C.o			 {corepath}PRISM.C
	{ShowProgress}
	{C} {corepath}PRISM.C {segPolygon} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}PRISM.C.o

{objpath}QUADRICS.C.o		 {corepath}QUADRICS.C
	{ShowProgress}
	{C} {corepath}QUADRICS.C {segPoly} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}QUADRICS.C.o

{objpath}QUATERN.C.o		 {corepath}QUATERN.C
	{ShowProgress}
	{C} {corepath}QUATERN.C {segJulia} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}QUATERN.C.o

{objpath}RADIOSIT.C.o		 {corepath}RADIOSIT.C
	{ShowProgress}
	{C} {corepath}RADIOSIT.C {segRender} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}RADIOSIT.C.o

{objpath}RAD_DATA.C.o		 {corepath}RAD_DATA.C
	{ShowProgress}
	{C} {corepath}RAD_DATA.C {segRender} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}RAD_DATA.C.o

{objpath}RAY.C.o			 {corepath}RAY.C
	{ShowProgress}
	{C} {corepath}RAY.C {segRender} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}RAY.C.o

{objpath}RENDER.C.o			 {corepath}RENDER.C
	{ShowProgress}
	{C} {corepath}RENDER.C {segRender} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}RENDER.C.o

{objpath}SOR.C.o		 {corepath}SOR.C
	{ShowProgress}
	{C} {corepath}SOR.C {segPolygon} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}SOR.C.o

{objpath}SPHERES.C.o		 {corepath}SPHERES.C
	{ShowProgress}
	{C} {corepath}SPHERES.C {segSphere} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}SPHERES.C.o

{objpath}SUPER.C.o		 {corepath}SUPER.C
	{ShowProgress}
	{C} {corepath}SUPER.C {segPoly} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}SUPER.C.o

{objpath}TARGA.C.o			{corepath}TARGA.C
	{ShowProgress}
	{C} {corepath}TARGA.C {segTarga} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}TARGA.C.o

{objpath}TEXTURE.C.o		{corepath}TEXTURE.C
	{ShowProgress}
	{C} {corepath}TEXTURE.C {segTex} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}TEXTURE.C.o

{objpath}TOKENIZE.C.o		{corepath}TOKENIZE.C
	{ShowProgress}
	{C} {corepath}TOKENIZE.C {segTokenize} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}TOKENIZE.C.o

{objpath}TORUS.C.o		{corepath}TORUS.C
	{ShowProgress}
	{C} {corepath}TORUS.C {segPoly} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}TORUS.C.o

{objpath}TRIANGLE.C.o		 {corepath}TRIANGLE.C
	{ShowProgress}
	{C} {corepath}TRIANGLE.C {segTri} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}TRIANGLE.C.o

{objpath}TRUETYPE.C.o		 {corepath}TRUETYPE.C
	{ShowProgress}
	{C} {corepath}TRUETYPE.C {segTT} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}TRUETYPE.C.o

{objpath}TXTTEST.C.o		{corepath}TXTTEST.C
	{ShowProgress}
	{C} {corepath}TXTTEST.C {segPrim2} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}TXTTEST.C.o

{objpath}USERIO.C.o			 {corepath}USERIO.C {corepath}USERIO.H
	{ShowProgress}
	{C} {corepath}USERIO.C {segParse} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}USERIO.C.o

{objpath}VBUFFER.C.o		 {corepath}VBUFFER.C
	{ShowProgress}
	{C} {corepath}VBUFFER.C {segLBuffer} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}VBUFFER.C.o

{objpath}VLBUFFER.C.o		 {corepath}VLBUFFER.C
	{ShowProgress}
	{C} {corepath}VLBUFFER.C {segLBuffer} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}VLBUFFER.C.o

{objpath}WARPS.C.o		{corepath}WARPS.C
	{ShowProgress}
	{C} {corepath}WARPS.C {segTex} {MyCOptions} {CAltOptions} {CInc2Folders} {PerfCmpOptions} -o {objpath}WARPS.C.o

# <eof>
