/*	
	HASMain.h from Hsoi's App Shell, 1995-1997 John C. Daub, All rights reserved.
	
	This is the "main" header file for Hsoi's App Shell.  It contains things specific
	to being a header file for HASMain.c (like the relevant function prototypes), but
	this file also contains various "global" things like constants, macros, resource
	ID's, etc. that are used throughout the shell.  Just ain't a better place to
	put 'em. :-)
	
	The naming conventions in here (for the most part) work like this (the "prefix"
	to each constant):
	
		r			resource (a "catch-all" for resources not fitting other prefix catagories)
		str			a string
		err			an error string
		i			an item (a menu item or a dialog/alert/window item)
		m			a menu
		k			a constant (usually just some "number")
*/


#ifndef __HSOIS_APP_SHELL__
#define __HSOIS_APP_SHELL__

#ifndef __TYPES__
#include <Types.h>
#endif
#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

/*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*\
 *																					 *
 *				All (well, most) of the #defines used in Hsoi's App Shell			 *
 *																					 *
\*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*/

#if defined(powerc) || defined (__powerc)
#pragma options align=mac68k
#endif

/*
 *	Debug/Testing macros
 */

// check against this macro for conditional debugging/testing throughout the
// shell.  if 1, use the debug code.  when you're ready to ship a final version
// of your product, #define HAS_DEBUG 0

#define	HAS_DEBUG	1

// and some other macros that you might find useful to use

// you might find it useful to change the value of CHECKPOINTS and ASSERTIONS in
// #if HAS_DEBUG around while you're working, espcially the CHECKPOINTS one (else
// that might cause your code to "break" a lot.  However, it'd be best to leave
// the values in the #else at 0 so as to ensure tests code doesn't get included
// in the end product.

#if HAS_DEBUG

#define	CHECKPOINTS		0	// i don't use checkpoints that often....
#define	ASSERTIONS		1	// ... but i like assertions

#else

#define	CHECKPOINTS		0
#define	ASSERTIONS		0

#endif


#if ASSERTIONS
	#define ASSERT_SET_NIL(lvalue)   lvalue = nil
#else
	#define ASSERT_SET_NIL(lvalue)
#endif


#if ASSERTIONS
	#define		ASSERT(x,y)		if (!(x)) {DebugStr("\p" y);}
#else
	#define		ASSERT(x,y)
#endif ASSERTIONS

#if CHECKPOINTS
	#define CKPT(x)		DebugStr( "\p" x )
#else
	#define CKPT(x)
#endif CHECKPOINTS



/*
 *	Various application specific constants
 */

/*	
	NOTE ABOUT CREATOR_TYPEs:

	The signature of the app is 'Hsoi'.  That's what makes it uniquely identifyable by the
	Finder.  All files created by/from/for Hsoi's App Shell will have this signature attached
	to it as the creator type, except for one file:  the preferences file.
	
	Why is the prefs different?  Well, it's due to that message string resource (see WritePrefsFile()
	in Preferences.c for more info).  If the creator type of the file is found, the app launches
	(so if the prefs have a 'Hsoi' creator type, the Demo app will launch).  If the creator
	type of the file is NOT found and there is a message string, that string will be displayed
	instead giving the user more information.
	
	So, this requires the prefs file to have it's own creator type, and one unique to it.
	
	I've registered all my creator types with Apple so as to prevent any sort of conflicts
	(and you should register with Apple any creator/file/resource types that you make) and
	to ensure the showing of the message string should the prefs file be clicked on.
*/
 
// NOTE:  APP_VERS_STR is just a pascal string equivalent of the app version.  the
// APP_VERS_NUM is in the NumVersion format (which is part of the VersRec - the struct
// that makes up the 'vers' resource...see Types.h and IM:Toolbox Essentials:7-31)
// FYI: 0x20 == development, 0x40 == alpha, 0x60 == beta, 0x80 == release

#define	APP_NAME			"Hsoi's App Shell"
#define APP_VERS_STR		"\p1.0a3" // note, it's a Pascal string
#define APP_VERS_NUM		0x01004003
#define APP_COPYRIGHT		"\p1995-97 John C. Daub. All rights reserved."
#define APP_SIGNATURE		'Hsoi'
#define APP_CREATOR_TYPE	APP_SIGNATURE

#define PREF_CREATOR_TYPE	'HSoi'
#define	PREF_FILE_NAME		"\pHsoi's App Shell Demo Prefs"	// new prefs file name
#define	PREF_FILE_NAME_OLD	"\pHAS Demo Preferences"		// old prefs file name (don't use this anymore)

#define DEFAULT_USERNAME		"\pFred Flintstone"
#define DEFAULT_ORGANIZATION	"\pSlate Rock and Gravel"

/*
 *	A few definitions of things for convenience
 */

#ifndef NIL
#define NIL					nil 	// "nil" is defined in Types.h
#endif
#define NIL_STRING			"\p"

#define MAXLONG				0x7FFFFFFF
#define MAXSHORT			0x7FFF
#define MINSHORT			0x8000

#define MOVE_TO_FRONT		(WindowRef)-1L
#define NOT_A_NORMAL_MENU	-1

#define ON					1
#define OFF					0
#define YES					1
#define NO					0

// you shouldn't need to #define these (much of these sorts of things are #defined
// in Types.h), but just in case, and just in case you like using CAPS instead of
// lower case....(i used to)

#ifndef true
#define true 1
#endif
#ifndef false
#define false 0
#endif
#ifndef TRUE
#define TRUE true
#endif
#ifndef FALSE
#define FALSE false
#endif




/*
 *	some macros for efficiency
 */
 
#define HiWrd(aLong)		(((aLong) >> 16) & 0xFFFF )
#define LoWrd(aLong)		((aLong) & 0xFFFF )

#define MIN(a, b) ((a) < (b) ? (a) : (b) )
#define MAX(a, b) ((a) > (b) ? (a) : (b) )

// these #define's taken from WASTEIntf.h

#define BTST( FLAGS, BIT )	(((FLAGS) & (1L << (BIT))) ? 1 : 0)
#define BSET( FLAGS, BIT )  ((FLAGS) |= (1L << (BIT)))
#define BCLR( FLAGS, BIT )  ((FLAGS) &= (~(1L << (BIT))))

#define ABS(A) ((A) > 0 ? (A) : -(A))

#define BSL(A, B)	(((long)A) << (B))
#define BSR(A, B)	(((long)A) >> (B))
#define BOR(A, B)	( A | B )
#define BAND(A, B)	( A & B )

// used to get a listing of all the open windows

#define mWindList (WindowRef) *((WindowRef *)LMGetWindowList())


/*
 *	resource types, clipboard types, and file types
 */

#define TYPE_DESK_ACCESSORY		'DRVR'
#define TYPE_FONT				'FONT'
#define TYPE_PICTURE			'PICT'
#define TYPE_SOUP				'SOUP'
#define TYPE_STYLE				'styl'
#define TYPE_TEXT				'TEXT'
#define TYPE_TEXT_READ_ONLY		'ttro'
#define TYPE_MENU_COLOR_TABLE	'mctb'	// resource type of a color menu
#ifndef TYPE_ANIMATED_CURSOR
#define TYPE_ANIMATED_CURSOR	'acur'
#endif
#define TYPE_SMALL_ICON			'SICN'
#define TYPE_SOUND				'snd '
#define	TYPE_PREFERENCES		'pref'

/*
 *	virtual char codes (NOT key codes)..as in: theKey = event->message & charCodeMask
 */

#define	kEnterKey		0x03
#define kTabKey			0x09
#define kReturnKey		0x0D
#define kBackSpace		0x08	// the backspace/delete key
#define kEscKey			0x1B
#define kLeftArrow		0x1C
#define kRightArrow		0x1D
#define kUpArrow		0x1E
#define kDownArrow		0x1F
#define kDeleteKey		0x7F

#define kPageUpKey		0x0B
#define kPageDownKey	0x0C
#define kHomeKey		0x01
#define kEndKey			0x77
#define kClearKey		0x1B // note: same charCode on "clear" and "esc" but keyCode for
							 //	clear is 0x35 and esc is 0x47
#define kForwardDelKey	0x7F
#define	kHelpKey		0x05



/*
 *	virtual KEY codes...as in:  theKey = (event->message & keyCodeMask ) >> 8
 */

#define	keyPgUp			0x74
#define keyPgDn			0x79
#define keyHome			0x73
#define keyEnd			0x77

#define		keyF1		0x7A
#define		keyF2		0x78
#define		keyF3		0x63
#define		keyF4		0x76

#define	keySpaceBar		0x31


/*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*\
 *																					 *
 *			All (well, again, most) of the constants used in Hsoi's App Shell		 *
 *																					 *
\*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*/

/*
	NOTE!

	I use enums for constants instead of #defines cause you can't redefine an enum, so
	it's a little safer to use enums
*/ 



/*
 *	coordinates for the disk initialization dialog
 */
 
enum {
	kDITop	=			80,
	kDILeft	=			112
};



/*
 *	Various constants used with window sizing
 */
 
enum {
	kLeftMargin	=		10,		//	left margin for window drawing
	kMargin		=		36,		// margins for printing...i'd like to have .5" margins
								//	and at 72dpi, that'd be 36 (I hope)
	kScrollBarWidth	=	16,		// width of a scroll bar
	kTitleBarHeight	=	20,		// usual height of a window title bar
	kTextMargin		=	3,		// indent of text rect from window port rect
	kScrollDelta	=	11		// pixels to scroll when the scroll bar arrow is clicked
};



/*
 *	Constants for the WindowPeek->windowKind field
 *  (userKind (8), dialogKind (2), and "DA kind" ( < 0 ) are defined in Windows.h,
 *	and I hope these don't interfere with any already defined System #'s.  kClipboardKind
 *	just needs to be more than userKind)
 */

// as a note of learning....at one time i wanted to keep track of all my dialogs/windows
// by assigning each one (except for Alerts and stuff) it's own unique windowKind.
// this is find for windows, but NOT fine to do for dialogs.  dialogs MUST have a
// windowKind of dialogKind (2) else the Dialog Manager doesn't know to handle them
// (and then they get treated like windows and YOU have to manage them).  save yourself
// the hassle...don't change the windowKind field of dialogs.  if you need to keep
// track of these sorts of things, have a global DialogRef for each dialog

enum {
	kClipboardKind		=	10,
//	kHelpDialogKind		=	11,	// not used
	kDocumentKind		=	12
//	kFindDialogKind		=	13	// not used
};


/*
 *	the Scrap threshold used during initialization
 */
 
enum {
	kScrapThreshold	=	4096	// 4 * 1024
};



/*
 *	some enumeration types used for closing a window and/or quitting the application
 */

typedef enum {closingWindow, closingApplication} ClosingOption;
typedef enum {savingYes, savingNo, savingAsk} SavingOption;



/*
 *	possible values for HandleOpenDocument refCon parameter
 */

enum {
	kDoOpen		=	0,
	kDoPrint	=	1
};




/*
 *	values for HiliteControl hiliteState parameter
 */

enum {
	kCtlActive		=	0,		// active control
	kCtlHilited		=	1,		// highlighted active control
	kCtlBackground	=	254,	// inactive control in an inactive window
	kCtlInactive	=	255		// inactive control in the active window
};



/*
 *	constants to pass for DoError for the errType
 */

enum {
	kErrNote	=	1,	// note and generic are the "same" thing
	kErrGeneric	=	2,
	kErrCaution	=	3,
	kErrStop	=	4,	// stop and death are similar, but death will call ExitToShell()
	kErrDeath	=	5
};


/*
 *	constants used in HsoiDoClose()
 */

enum {
	kButtonSave		=	1,
	kButtonCancel	=	2,
	kButtonDontSave	=	3
};

// max number of open document windows

enum {
	kMaxNumberOfOpenWindows		=	25
};



/*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*\
 *																					 *
 *	 All (well, again, most) of the resource ID numbers used in Hsoi's App Shell	 *
 *																					 *
\*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*/




/*
 *	'cicn' resource numbers
 */

enum {
	rStopIconID				=	0,
	rNoteIconID				=	1,
	rCautionIconID			=	2,
	rIconIDHomer			=	128

};


/*
 *	'CNTL' resource numbers
 */

enum {
	rScrollBarTemplate		=	1000 // used as a template to create the window scrollbars
};



/*
 *	'DITL' resource numbers
 */

enum {
	rCoolAboutDITL				=	2000,
	iCoolAboutBkgrnd			=	1,
	iCoolAboutLogo				=	2,
	iCoolAboutVersion			=	3,
	iCoolAboutText				=	4
};



/*
 * 'DLOG' resource numbers, and their associated 'DITL' item numbers
 */


enum {
	rPrintStatusDialog			=	132
};


enum {
	rPrefsGeneralDLOG			=	400,
	iPrefsGenCreateWIND			=	4,
	iPrefsGenDoSplash			=	5,
	iPrefsGenHelpSound			=	6,
	iPrefsGenStartupSpeech		=	7
};

enum{
	rPrefsTextDLOG				=	401,
	iPrefsTextFontPopup			=	4,
	iPrefsTextSizeEditBox		=	6,
	iPrefsTextSizePopup			=	7,
	iPrefsTextPlainCheck		=	8,
	iPrefsTextBoldCheck			=	9,
	iPrefsTextItalicCheck		=	10,
	iPrefsTextUnderlineCheck	=	11,
	iPrefsTextOutlineCheck		=	12,
	iPrefsTextShadowCheck		=	13,
	iPrefsTextCondenseCheck		=	14,
	iPrefsTextExtendCheck		=	15,
	iPrefsTextGetColor			=	17,
	iPrefsTextLeftRadio			=	18,
	iPrefsTextRightRadio		=	19,
	iPrefsTextCenterRadio		=	20,
	iPrefsTextFullRadio			=	21,
	iPrefsTextTabHooks			=	22,
	iPrefsTextAutoScroll		=	23,
	iPrefsTextOutlineHilite		=	24,
	iPrefsTextReadOnly			=	25,
	iPrefsTextIntCutAndPaste	=	26,
	iPrefsTextDragAndDrop		=	27,
	iPrefsTextOffscreenDrawing	=	28,
	iPrefsTextStyleUserItem		=	29,
	iPrefsTextAlignUserItem		=	30,
	iPrefsTextFeaturesUserItem	=	31,

	kPrefsTextFirstRadio		=	iPrefsTextLeftRadio,
	kPrefsTextLastRadio			=	iPrefsTextFullRadio,
	kPrefsTextDefaultRadio		=	iPrefsTextLeftRadio // bad kludge for now until i get a "default align" radio button
};

enum {
	rPrefsNMDLOG				=	402,
	iPrefsNMIcon				=	4,
	iPrefsNMDiamond				=	5,
	iPrefsNMSound				=	6,
	iPrefsNMAlert				=	7
};

enum {
	rPrefsPrintDLOG				=	403,
	iPrefsPrintPageNum			=	4,
	iPrefsPrintDblSpace			=	5,
	
	iPrefsPrintTextTop			=	6,		// 6-9: edit texts for user input
	iPrefsPrintTextBottom		=	7,
	iPrefsPrintTextLeft			=	8,
	iPrefsPrintTextRight		=	9,
	
	iPrefsPrintTop				=	10,		// 10-13: static text labels
	iPrefsPrintBottom			=	11,
	iPrefsPrintLeft				=	12,
	iPrefsPrintRight			=	13,
	
	iPrefsPrintTopUpUserItem	=	14,
	iPrefsPrintTopDownUserItem	=	15,
	iPrefsPrintTopArrowIcon		=	16,
	
	iPrefsPrintBotUpUserItem	=	17,
	iPrefsPrintBotDownUserItem	=	18,
	iPrefsPrintBotArrowIcon		=	19,
	
	iPrefsPrintLeftUpUserItem	=	20,
	iPrefsPrintLeftDownUserItem	=	21,
	iPrefsPrintLeftArrowIcon	=	22,
	
	iPrefsPrintRightUpUserItem	=	23,
	iPrefsPrintRightDownUserItem	= 24,
	iPrefsPrintRightArrowIcon	=	25,
	
	iPrefsPrintLineItem			=	26, // draws the little separater line
	
	iPrefsPrintMarginFrame		=	27, // draws the frame around the margins stuff
	
	iPrefsPrintMarginsText		=	28
};


enum {
	iPrefsTopicPopup			=	3
};



/*
 *	'Hso?' resource numbers
 */

enum {
	rShowHelpInfo				=	128
};



/*
 *	'ICON' resource numbers
 */

enum {
	rBaseSpinIcon				=	129,
	rUpDownArrowIcon			=	1001,	// up/down arrow icon/control for print prefs margins
	rUpArrowIcon				=	1002,	// same, but with up arrow "hilited"
	rDownArrowIcon				=	1003	// same, but with down arrow "hilited"
};



/*
 *	'MBAR' resource numbers
 */

enum {
	rMenuBar				=	128
};



/*
 *	'MENU' resource numbers (and their associated item numbers)
 */

enum {
	mApple					=	128,
	iAbout					=	1
};


enum {
	mFile					=	129,
	iNew					=	1,
	iOpen					=	2,
	//------- is item 3
	iClose					=	4,
	iSave					=	5,
	iSaveAs					=	6,
	iRevert					=	7,
	//------- is item 8	
	iPageSetup				=	9,
	iPrint					=	10,
	//------- is item 11	
	iPreferences			=	12,
	//------- is item 13	
	iQuit					=	14
};

enum {
	mEdit					=	130,
	iUndo					=	1,
	//------- is item 2
	iCut					=	3,
	iCopy					=	4,
	iPaste					=	5,
	iClear					=	6,
	iSelectAll				=	7,
	//----- is item 8
	iShowClipboard			=	9
};

enum {
	mText					=	131,
	iFont					=	1,
	mFont					=	100,
	iSize					=	2,
	mSize					=	101,
	iSize9					=	1,
	iSize10					=	2,
	iSize12					=	3,
	iSize18					=	4,
	iSize24					=	5,
	iSize36					=	6,
	iSize48					=	7,
	//----- separator line, item 8
	iSizeSmaller			=	9,
	iSizeLarger				=	10,
	//----- separator line, item 11
	iSizeOther				=	12,
	
	kSizeMenuLast			=	iSize48
};	

enum {
	iStyle					=	3,
	mStyle					=	102,
	iPlain					=	1,
	iBold					=	2,
	iItalic					=	3,
	iUnderline				=	4,
	iOutline				=	5,
	iShadow					=	6,
	iCondensed				=	7,
	iExtended				=	8,
	kPlainStyle				=	0,
	
	iJustification			=	4,
	mJustification			=	103,
	iDefaultJust			=	1,
	//--------- is item 2
	iLeftAlign				=	3,
	iCenterAlign			=	4,
	iRightAlign				=	5,
	iFullAlign				=	6,
	
	iColor					=	5,
	mColor					=	105,
	iColorBlack				=	1,
	iColorRed				=	2,
	iColorGreen				=	3,
	iColorBlue				=	4,
	iColorCyan				=	5,
	iColorMagenta			=	6,
	iColorYellow			=	7,
	//---------- is item 8
	iColorOther				=	9,

// ------ separator line is item 6 in mText

	iFeatures				=	7,
	mFeatures				=	106,
	iTabHooks				=	1,
	iAutoScroll				=	2,
	iOutlineHilite			=	3,
	iReadOnly				=	4,
	iIntelligentCutAndPaste	=	5,
	iDragAndDropEditing		=	6,
	iOffscreenDrawing		=	7
};


enum {
	mDialogs				=	132,
	iModal					=	1,
	iModeless				=	2,
	iMovable				=	3
};

enum {
	mWindows				=	133,
	iTileWindows			=	1,
	iStackWindows			=	2,
	iWindowMenuSeperator	= 	3, // necessary to have this. see HsoiDoMenuCommand() for why
	iFirstWindow			=	4
};	


enum {
	mSound					=	134,
	iRecordNewSnd			=	1,
	iPlaySnd				=	2,
	iStopPlayingSnd			=	3,
	//---- is item 4
	iSpeakAll				=	5,
	iSpeakFromCursor		=	6,
	iSpeakSelection			=	7,
	iStopSpeaking			=	8,
	iPauseSpeaking			=	9,
	//---- item 10
	iTurnSpeechOnOff		=	11,
	//---- item 12
	iAutoHiliting			=	13,
	mHiliting				=	107,	// this is the Auto-Hiliting sub menu
		iHilitingNone		=	1,
		iHilitingWord		=	2,
		iHilitingSentence	=	3,
	iSpeechOptions			=	14
};


/*
 *	'PICT' resource numbers
 */

enum {
	rLogoPICT						=	1000,
	rVersionPICT					=	1001,
	rCoolAboutBkgrndPICT			=	2000
};



/*
 *	'SICN' resource numbers
 */

enum {
	rLeftJustIcon			=	280,
	rCenterJustIcon			=	281,
	rRightJustIcon			=	282,
	rFullJustIcon			=	283,
	rNotifyIcon				=	400
};



/*
 *	'snd ' resource numbers
 */

enum {
	rSoundIDHomer				=	128,
	rSoundHelp					=	129,

	rAboutSoundWow				=	5000,
	rAboutSoundSLM				=	5001,
	rAboutSoundUhh				=	5002,
	rAboutSoundCool				=	5003,

	kAboutSoundMax				=	rAboutSoundCool,
	kAboutSoundMin				=	rAboutSoundWow
};



/*
 *	'STR ' resource numbers
 */

enum {
	strPutFilePrompt			=	128,
	strHelpMenuItemText			=	129,
	strMessageString			=	130,
	strAppNameString			=	-16396
};




/*
 *	'STR#' resource numbers, and the numbers for the strings within them
 */

enum {
	rErrorStrings			=	128,
	errWimpyROMs			=	1,
	errWimpySystem			=	2,
	errWeirdSystem			=	3,
	errNoMenuBar			=	4,
	errNoFSSpec				=	5,
	errCantDrawDialog		=	6,
	errCantLoadSound		=	7,
	errCantCreateWindow		=	8,
	errNoAppleEvents		=	9,
	errAEInstallFail		=	10,
	errNoColorQD			=	11,
	errNoDragAndDrop		=	12,
	errNoTextServices		=	13,
	errNoAliasMgr			=	14,
	errNoEditionManager		=	15,
	errFailedEditionInit	=	16,
	errFailedTSMInit		=	17,
	errFailDragTrackInit	=	18,
	errFailDragReceiveInit	=	19,
	errFailObjectInit		=	20,
	errFailGetResource		=	21,
	errFailResolveAlias		=	22,
	errNoNotification		=	23
};

enum {
	rUndoStrings			=	129,
	strCantUndo				=	1,
	strUndo					=	2,
	strRedo					=	3,
	strUndoTyping			=	4,
	strRedoTyping			=	5,
	strUndoCut				=	6,
	strRedoCut				=	7,
	strUndoPaste			=	8,
	strRedoPaste			=	9,
	strUndoClear			=	10,
	strRedoClear			=	11,
	strUndoDrag				=	12,
	strRedoDrag				=	13,
	strUndoStyleChange		=	14,
	strRedoStyleChange		=	15
};

enum {
	rQuitCloseStrings		=	130,
	strClosing				=	1,
	strQuitting				=	2
};



enum {
	rPrintingStrings			=	132,
	strPrintBeginning			=	1,
	strPrintMiddle				=	2,
	strPrintEnd					=	3,
	strPrintError				=	4
};

enum {
	rOtherSizeStrings				=	133,
	strOtherSizeOtherOnly			=	1,
	strOtherSizeOtherSpace			=	2,
	strOtherSizeLeftParen			=	3,
	strOtherSizeRightParen			=	4,
	strOtherSizeMixed				=	5
};


enum {
	rFeaturesMenuStrings			=	134,
	strFeatureEnableTabHooks		=	1,
	strFeatureDisableTabHooks		=	2,
	strFeatureEnableAutoScroll		=	3,
	strFeatureDisableAutoScroll		=	4,
	strFeatureEnableOutlineHilite	=	5,
	strFeatureDisableOutlineHilite	=	6,
	strFeatureEnableReadOnly		=	7,
	strFeatureDisableReadOnly		=	8,
	strFeatureEnableCutAndPaste		=	9,
	strFeatureDisableCutAndPaste	=	10,
	strFeatureEnableDragAndDrop		=	11,
	strFeatureDisableDragAndDrop	=	12,
	strFeatureCantDragAndDrop		=	13,
	strFeatureEnableOffscreen		=	14,
	strFeatureDisableOffscreen		=	15
};

enum {
	rGetColorStrings				=	135,
	strGetTextColor					=	1,
	strDefaultTextColor				=	2
};


enum {
	rCoolAboutBoxMenuItemStrings	=	136,
	strAboutHAS						=	1
};

enum {
	rPrefsDialogStrings				=	137,
	strPrefsDragAndDrop				=	1,
	strPrefsCantDragAndDrop			=	2
};


enum {
	rUntitledWindowStrings			=	138,
	strUntitledWindow				=	1
};

enum {
	rNotificationManagerStrings		=	139,
	strNMBringToFront				=	1
};

enum {
	rClipboardWindowStrings			=	140,
	strClipboardWindowTitle			=	1,
	strClipboardEmpty				=	2
};

enum {
	rMovableModalStrings		=	202,
	strDoingLongTask			=	1
};

enum {
	rReadErrorStrings			=	10000,
	strRdTxFSpOpenDF			=	1,
	strRdTxGetEOF				=	2,
	strRdTxSetFPos				=	3,
	strRdTxNewHandle			=	4,
	strRdTxFSRead				=	5,
	strRdTxFSClose				=	6,
	strRdTxGetStyleH			=	7,
	strRdTxGetSoupH				=	8,
	strRdTxCloseResFile			=	9,
	strRdTxWEInsert				=	10
};


enum {
	rWriteErrorStrings			=	10001,
	strWrTxFFSpGetInfo			=	1,
	strWrTxFFSpDelete			=	2,
	strWrTxFFSpCrRsF			=	3,
	strWrTxFFSpSetFInfo			=	4,
	strWrTxFFSpOpenDF			=	5,
	strWrTxFSetEOF				=	6,
	strWrTxFSetFPos				=	7,
	strWrTxFFSWrite				=	8,
	strWrTxFFSpOpenResFile		=	9,
	strWrTxFstylMemErr			=	10,
	strWrTxFsoupMemErr			=	11,
	strWrTxFWECopyRange			=	12,
	strWrTxFstylAddRes			=	13,
	strWrTxFsoupAddRes			=	14,
	strWrTxFstylWriteRes		=	15,
	strWrTxfsoupWriteRes		=	16,
	strWrTxFindFolder			=	17,
	strWrTxFSMakeFSSpec			=	18
};


enum {
	rWindowErrorStrings			=	10002,
	strCrWindNewPtrClear		=	1,
	strCrWindGetNewWind			=	2,
	strCrWindWENew				=	3,
	strCrWindWESetInfo			=	4,
	strCrWindWESetInfoClick		=	5,
	strCrWindWESetInfoScroll	=	6,
	strCrWindWESetInfoDrag		=	7,
	strCrWindGetNewControlV		=	8,
	strCrWindLCAttachV			=	9,
	strCrWindGetNewControlH		=	10,
	strCrWindLCAttachH			=	11,
	strCrWindFSpGetFInfo		=	12,
	strCrWindkTypeText			=	13,
	strCrWindReadTextFile		=	14,
	strStdDlgFilterSetDef		=	15
};


/*
 *	'TEXT' resource numbers
 */

enum {
	rCoolAboutCreditsText		=		2000
};



/*
 *	'WIND' resource numbers
 */

enum {
	rMainWindow					=	128,
	rCoolAboutWIND				=	2000
};


//*******
// And before we go, the function prototypes for HASMain.c

void			main( void );
void			HsoiInitialize( void );
void			HsoiCheckForKosherSystem( void );
void			HsoiEventLoop( void );
void			HsoiDoMainLoopTasks( void );
OSErr			HsoiCleanUp( SavingOption );



#if defined(powerc) || defined (__powerc)
#pragma options align=reset
#endif


#endif // #define __HSOIS_APP_SHELL__