// Persistence of Vision Ray Tracer Scene Description File
// File: qtvrPanorama.pov
// Vers: 3
// Desc: Show how to design a QuickTime VR panoramic movie file.
//       After rendering this at 1248x384, you need to use
//       Apple's free application "Make QTVR Panorama" to
//       turn it into a panorama movie that can be displayed with
//       Apple's QTVR Player.
//
//       If you want better resolution, try any multiple of 4
//       for width, and multiple of 96 for height.  A good
//       high resolution QTVR scene would be 2016x756.
//
//       This scene is just a quick proof-of-concept, not a
//       full-fledged Myst environment!
// Date: 02/02/1997
// Auth: Eduard Schwan
// Enhancements: Stephen Andrusyszyn, balalaika@earthlink.net
//

#version 3

#include "colors.inc"
#include "metals.inc"

global_settings
{
  assumed_gamma 1.0
}

// ----------------------------------------
camera
{
  // Create a 360 degree camera view on its side
  cylinder 2     // sideways cylinder camera (along X axis)
  angle 360      // full 360 degree view

  right  1*x
  up     1/2*y  // wider angle
  sky    -x     // orient camera on its side for QTVR Pano tool
  location  <0.0, 0.0, 0.0>
  look_at   <0.0, 0.0, 1.0>
}


// ----------------------------------------
sky_sphere
{
  pigment
  {
    gradient y
    color_map { [0.0 color blue 0.5] [1.0 color rgb 1] }
  }
}

// ----------------------------------------
light_source
{
  0*x // light's position (translated below)
  color red 1.0  green 1.0  blue 1.0  // light's color
  translate <-100, 50, -50>
}


// ----------------------------------------
plane
{
  y, -1
  texture
  {
    pigment {checker color rgb 1 color blue 0.5 scale <0.5, 1, 2> }
    finish {ambient 0.2}
  }
}

// ----------------------------------------
// glass sphere (-z)
sphere
{
  0, 2
  translate <0,1,-6>
  texture
  {
    pigment {color rgb <0.5,1,1> filter 0.9}
    finish
    {
      specular 0.5 refraction 1 ior 1.4
      irid { 0.3 thickness 0.2 turbulence 0.7 }
    }
  }
}

// ----------------------------------------
// little marble (-x)
sphere
{
  0, 0.5
  texture
  {
    pigment{radial frequency 8 rotate 30*x}
    finish{specular 0.8}
  }
  translate <-3, -0.5, 0.5>
}

// ----------------------------------------
// tower of torii (+7x)
#declare tc = 0
#while (tc < 5)
  torus
  {
    1, 0.5  rotate 90*x
    rotate (tc*20)*y
    translate <12,tc*2,3>
    texture
    { T_Brass_1C }
 }
  #declare tc = tc+1
#end

// ----------------------------------------
// Gazebo in center, that we look out from (0)
// first, a rounded square +/-Z bore
#declare CylinderCluster = union
{
  cylinder {-z,+z,1}
  cylinder {-z,+z,1 translate -x-y}
  cylinder {-z,+z,1 translate +x-y}
  cylinder {-z,+z,1 translate -x+y}
  cylinder {-z,+z,1 translate +x+y}
  scale <0.4, 0.4, 1.1>
}
// now a box with the above cylinder set bored through it
#declare HollowBox = intersection
{
  box {-1, +1}
  object {CylinderCluster inverse}
  object {CylinderCluster rotate 90*y inverse}
  object {CylinderCluster rotate 90*x inverse}
}
#declare HBTex = texture
{
  pigment
  {
    crackle turbulence 0.3 scale 0.2  rotate 20*y
    color_map {[0.05 red 0.1] [0.1 rgb <.8,.6,.4>] [0.3 rgb 0.7]}
  }
  normal
  {
    crackle turbulence 0.3 scale 0.2  rotate 20*y bump_size 0.4
  }
  finish { ambient 0.2 specular 0.5 roughness 0.1 }
}
object { HollowBox texture{HBTex} }

// ----------------------------------------
// another gazebo off in the distance (+z)
object { HollowBox texture{HBTex} rotate 60*y translate 4*z}


